;;; $Id: zical.el,v 1.1 2004/02/22 21:55:15 noir Exp $
;;; Copyright (C) 2004 Hiroyuki KUROSAKI <noir@st.rim.or.jp>
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;;
;;; Utility elisp for ZiCal.
;;;
;;; Requirement:
;;;
;;; ZiCal
;;; http://noir.s7.xrea.com/archives/000059.html
;;;
;;; icalendar.el version 0.05 by Ulf Jasper
;;; http://de.geocities.com/ulf_jasper/emacs.html
;;;
;;; Installation:
;;;
;;; 1. Copy icalendar.el to the working directory.
;;;
;;; 2. Modify icalendar.el:
;;;
;;; patch -p0 < icalendar.el.diff
;;;
;;; 3. Copy the modified icalendar.el and zical.el to your load-path.
;;;
;;; 4. Add the lines to your ~/.emacs.
;;;
;;; (require 'zical)
;;; (setq zical-diary-file "~/zical-diary")
;;;
;;; 5. Add the line to your ~/diary.
;;;
;;; #include "~/zical-diary"
;;;
;;; Usage:
;;;
;;; 1. Connect Zaurus to PC via USB cable.
;;;
;;; 2. Run ZiCal on Zaurus and push "Start" button.
;;;
;;; 3. M-x zical-import-to-diary
;;;
;;; 4. M-x calendar
;;;
;;;
;;; Customize Calendar/Diary
;;;
;;; Reference: GNU Emacs Lisp Manual (info)
;;;
(require 'calendar)
(require 'icalendar)

(setq view-diary-entries-initially t)
(setq mark-diary-entries-in-calendar t)
(setq holidays-in-diary-buffer nil)
(setq number-of-diary-entries 7)

(add-hook 'diary-display-hook 'fancy-diary-display)
(add-hook 'list-diary-entry-hook 'sort-diary-entries t)
(add-hook 'list-diary-entries-hook 'include-other-diary-files)
(add-hook 'mark-diary-entries-hook 'mark-included-diary-files)

(setq calendar-time-display-form
      '(24-hours ":" minutes
		 (if time-zone " (") time-zone (if time-zone ")")))

;;;
;;; Utility Functions for ZiCal
;;;
(defcustom zical-url "http://192.168.129.201:8990/zaurus.ics"
  "URL to get iCalendar formatted file."
  :type 'string
  :group 'zical)
(defcustom zical-diary-file "~/zical-diary"
  "ZiCal specific diary file. If you choose fancy displayed diary,
  you can include it by '#include \"(filename)\"' line."
  :type 'string
  :group 'zical)
(defcustom zical-get-command "curl -s"
  "Command to get URL and output to stdout.
E.g. \"curl -s\", \"wget -q -O -\""
  :type 'string
  :group 'zical)
(defconst zical-output-buffer "*zaurus.ics*"
  "ZiCal temporary output buffer.")
(defconst zical-error-buffer "*zical error*"
  "ZiCal error buffer.")

(defun zical-read-from-url (&optional url)
  "Import iCalendar data from URL (default: zical-url) to
zical-output-buffer. Errors are displayed in zical-error-buffer.
Uses zical-get-command to get iCalendar data. Returns output buffer."
  (interactive)
  (setq url (or url zical-url))
  (when (interactive-p)
    (setq url (read-string "URL: " url)))
  (save-excursion
    (save-window-excursion
      (let ((output-buffer (get-buffer zical-output-buffer))
	    (coding-system-for-read 'utf-8-dos))
	(and output-buffer (set-buffer output-buffer) (erase-buffer))
	(shell-command (concat zical-get-command " " url)
		       zical-output-buffer zical-error-buffer))))
  zical-output-buffer)

(defun zical-import-to-diary (&optional url diary-file)
  "import iCalendar data from URL (default: zical-url) to DIARY-FILE."
  (interactive)
  (setq url (or url zical-url))
  (setq diary-file (or diary-file zical-diary-file))
  (when (interactive-p)
    (setq url (read-string "URL: " url))
    (setq diary-file (read-string "Diary File: " diary-file)))
  (let ((cbuf (current-buffer)))
    (let ((diary-buffer
	   (get-buffer (replace-regexp-in-string "^.*/" "" zical-diary-file))))
      (when diary-buffer
	(set-buffer diary-buffer)
	(set-buffer-modified-p nil)
	(kill-buffer diary-buffer)))
    (set-buffer (find-file zical-diary-file))
    (erase-buffer)
    (set-buffer (zical-read-from-url url))
    (icalendar-extract-ical-from-buffer zical-diary-file t)
    (or (string-equal
	 (buffer-name (current-buffer)) " *icalendar-errors*"))
    (switch-to-buffer cbuf)
    (message "Done.")))

(provide 'zical)
